#!/usr/bin/python2
# Emulates the behaviour of codesign --deep which is missing on OS X < 10.9

import os
import re
import subprocess
import sys

def SignPath(path, developer_id, deep=True):
  args = [
    'codesign',
    '--preserve-metadata=identifier,entitlements,resource-rules,requirements',
    '-s', developer_id,
    '-fv', path
  ]
  if deep:
    args.append('--deep')
  subprocess.check_call(args)

def main():
  if len(sys.argv) != 3:
    print 'Usage: %s <developer id> <app bundle>' % sys.argv[0]
    sys.exit(1)

  developer_id = sys.argv[1]
  app_bundle = sys.argv[2]

  for root, dirs, files in os.walk(app_bundle):
    for dir in dirs:
      if re.search(r'\.framework$', dir):
        SignPath(os.path.join(root, dir), developer_id)

    for file in files:
      if re.search(r'\.(dylib|so)$', file):
        SignPath(os.path.join(root, file), developer_id)
      elif re.match(r'(clementine-spotifyblob|clementine-tagreader|gst-plugin-scanner)', file):
        SignPath(os.path.join(root, file), developer_id)

  SignPath(app_bundle, developer_id, deep=False)

  # Verify the signatures are valid.
  subprocess.check_call([
      'codesign', '--verify', '--verbose=4', app_bundle])
  subprocess.check_call([
      'spctl', '--assess', '--verbose=4', app_bundle])


if __name__ == '__main__':
  main()



#!/usr/bin/python2
import codecs
import glob
import logging
import os
import polib
import re

PO_GLOB = 'src/translations/*.po'
DESKTOP_PATH = 'dist/org.clementine_player.Clementine.desktop'

class ConfigParser(object):
  """
  Better version of the ConfigParser from the stdlib that handles unicode.
  """

  SECTION_RE = re.compile('^\[(.*)\]\s*$')
  VALUE_RE = re.compile('^([^\s=]+)\s*=\s*(.*)')

  def __init__(self, encoding='utf-8'):
    self._sections = []
    self.encoding = encoding

  def read(self, filename):
    with codecs.open(filename, 'r', self.encoding) as handle:
      current_section = None

      for line in handle:
        match = self.SECTION_RE.match(line)
        if match:
          current_section = self._add_section(match.group(1))
          continue

        match = self.VALUE_RE.match(line)
        if match:
          current_section['items'].append(
              (match.group(1), match.group(2).strip()))
          continue

  def _add_section(self, section_name):
    section = {
      'name': section_name,
      'items': [],
    }
    self._sections.append(section)
    return section

  def _find_section(self, section_name):
    return [
      section
      for section in self._sections
      if section['name'] == section_name
    ][0]

  def _find_item_value(self, section, key):
    return [
      item[1]
      for item in section['items']
      if item[0] == key
    ][0]

  def sections(self):
    return [x['name'] for x in self._sections]

  def get(self, section_name, key):
    try:
      return self._find_item_value(self._find_section(section_name), key)
    except IndexError:
      return None

  def set(self, section_name, key, value):
    try:
      section = self._find_section(section_name)
    except IndexError:
      section = self._add_section(section_name)

    for index, (existing_key, _existing_value) in enumerate(section['items']):
      if existing_key == key:
        section['items'][index] = (key, value)
        return

    section['items'].append((key, value))

  def write(self, filename):
    with codecs.open(filename, 'w', self.encoding) as handle:
      for section in self._sections:
        handle.write('[%s]\n' % section['name'])
        for key, value in section['items']:
          handle.write('%s=%s\n' % (key, value))

        handle.write('\n')


def main():
  logging.basicConfig(level=logging.DEBUG,
                      format="%(asctime)s %(levelname)-7s %(message)s")

  config_parser = ConfigParser()
  config_parser.read(DESKTOP_PATH)

  shortcuts = [
    {
      'section_name': section_name,
      'original': config_parser.get(section_name, 'Name'),
    }
    for section_name in config_parser.sections()
    if section_name.endswith('Shortcut Group')
  ]

  for po_filename in glob.glob(PO_GLOB):
    language = os.path.splitext(os.path.basename(po_filename))[0]
    config_key = 'Name[%s]' % language
    po_file = polib.pofile(po_filename)

    logging.info('Reading language %s', po_filename)

    for shortcut in shortcuts:
      original = shortcut['original']
      entry = po_file.find(original)
      if entry is not None and entry.msgstr and entry.msgstr != original:
        logging.info('Setting %s for %s to %s',
            config_key, shortcut['section_name'], entry.msgstr)
        config_parser.set(shortcut['section_name'], config_key, entry.msgstr)

  config_parser.write(DESKTOP_PATH)


if __name__ == '__main__':
  main()



#!/usr/bin/python2

from subprocess import *
from sys import *
from os import rename, remove
from datetime import *

def pretty_years(s):

	l = list(s)
	l.sort()

	start = None
	prev = None
	r = []

	for x in l:
		if prev is None:
			start = x
			prev = x
			continue

		if x == prev + 1:
			prev = x
			continue

		if prev == start:
			r.append("%i" % prev)
		else:
			r.append("%i-%i" % (start, prev))

		start = x
		prev = x

	if not prev is None:
		if prev == start:
			r.append("%i" % prev)
		else:
			r.append("%i-%i" % (start, prev))

	return ", ".join(r)

def order_by_year(a, b):

	la = list(a[2])
	la.sort()

	lb = list(b[2])
	lb.sort()

	if la[0] < lb[0]:
		return -1
	elif la[0] > lb[0]:
		return 1
	else:
		return 0

def gen_copyrights(f):

	commits = []
	data = {}
        copyrights = []

	for ln in Popen(["git", "blame", "--incremental", f], stdout=PIPE).stdout:

		if ln.startswith("filename "):
			if len(data) > 0:
				commits.append(data)
			data = {}

		elif ln.startswith("author "):
			data["author"] = ln[7:].strip()

		elif ln.startswith("author-mail <"):
			data["author-mail"] = ln[12:].strip()

		elif ln.startswith("author-time "):
			data["author-time"] = ln[11:].strip()

		elif ln.startswith("author-tz "):
			data["author-tz"] = ln[9:].strip()

        with open(f,'r') as fi:
                fil = fi.readlines()
                for i in fil:
                        if -1 < i.find("Original Author"):
                                da = i.split("  ")
                                print f
                                copyrights.append("   Copyright %s, %s %s\n" % (da[3].strip(), da[1], da[2]))

        by_author = {}

	for c in commits:
		try:
			n =  by_author[c["author"]]
		except KeyError:
			n = (c["author"], c["author-mail"], set())
			by_author[c["author"]] = n

		# FIXME: Handle time zones properly
		year = datetime.fromtimestamp(int(c["author-time"])).year

		n[2].add(year)

	for an, a in list(by_author.iteritems()):
		for bn, b in list(by_author.iteritems()):
			if a is b:
				continue

			if a[1] == b[1]:
				a[2].update(b[2])

				if by_author.has_key(an) and by_author.has_key(bn):
					del by_author[bn]

	copyright = list(by_author.itervalues())
	copyright.sort(order_by_year)

	for name, mail, years in copyright:
		copyrights.append("   Copyright %s, %s %s\n" % (pretty_years(years), name, mail))
        return copyrights

def change_file(filename):
        content=[]
        out=[]
        extended=0
        ends=0
        with open(filename, "r") as fi:
                content=fi.readlines()

        copyrights=gen_copyrights(filename)

        if -1 == content[0].find("/* This file is part of Clementine."):
                print("File {} have no Clementine copyright info".format(filename))
                return 0

        for i in content:
                if i.find("*/") != -1:
                        ends = 1
                if i.find("Copyright ") != -1:
                        if not extended:
                                out.extend(copyrights)
                                extended = 1

                        if not ends:
                                continue
                else:
                        out.append(i)

        with open(filename+'_tmp', "w") as fi:
                fi.writelines(out)
        rename(filename+'_tmp', filename)


if __name__ == "__main__":
        for files in argv[1:]:
                change_file(files)



#!/usr/bin/python2
import rpm
import subprocess

deb_versions = [
  "1.0",
  "1.0.1",
  "1.0.2",
  "1.0.a",
  "1.0~rc1",
  "1.0~rc2",
  "1.0-506-g044287b",
  "1.0-1-g044287b",
]

rpm_versions = [
  ("1.0", "1"),
  ("1.0.1", "1"),
  ("1.0.2", "1"),
  ("1.0", "0.rc1"),
  ("1.0", "0.rc2"),
  ("1.0", "2.506-g044287b"),
]

def compare_deb(left, right):
  if subprocess.call(["dpkg", "--compare-versions", left, "lt", right]) == 0:
    return -1
  if subprocess.call(["dpkg", "--compare-versions", left, "eq", right]) == 0:
    return 0
  if subprocess.call(["dpkg", "--compare-versions", left, "gt", right]) == 0:
    return 1

  raise Exception("Couldn't compare versions %s %s" % (left, right))


def compare_rpm(left, right):
  return rpm.labelCompare(("1", left[0], left[1]), ("1", right[0], right[1]))

def sort_and_print(l, cmp):
  for v in sorted(l, cmp=cmp):
    if isinstance(v, tuple):
      print "%s-%s" % v
    else:
      print v
  print


sort_and_print(deb_versions, compare_deb)
sort_and_print(rpm_versions, compare_rpm)



#!/usr/bin/env python2
import argparse
import difflib
import os
import subprocess
import sys
import urllib2


def main():
  parser = argparse.ArgumentParser(
      description='Reformats C++ source files that have changed from a given '
                  'git ref.')
  parser.add_argument('--url',
      default='https://clang.clementine-player.org/format',
      help='a URL of a Clang-in-the-cloud service')
  parser.add_argument('--ref', default='origin/master',
      help='the git-ref to compare against')
  parser.add_argument('--extension', action='append', metavar='EXT',
      default=['cc', 'cpp', 'h', 'c', 'cxx', 'm', 'mm'],
      help='file extensions to reformat')
  parser.add_argument('-i', dest='inplace', action='store_true',
      help='edit files inplace instead of showing a diff')
  parser.add_argument('--files', nargs='*', metavar='FIL',
      default=[],
      help='get files as arguments instead of git')
  args = parser.parse_args()

  try:
    root_dir = subprocess.check_output([
        'git', 'rev-parse', '--show-toplevel']).strip()
  except subprocess.CalledProcessError:
    # Probably we were not called from a git working directory, just ignore this
    # error.
    return

  changed_files = subprocess.check_output([
      'git', 'diff-index', args.ref, '--name-only']).splitlines()

  if not changed_files:
    print >> sys.stderr, 'No changes from %s' % args.ref

  if not args.files and not changed_files:
    print >> sys.stderr, "Use --files to select files for reformat"
    return

  if args.files:
    changed_files = args.files

  for filename in changed_files:
    if not os.path.splitext(filename)[1][1:] in args.extension:
      continue

    path = os.path.join(root_dir, filename)
    if not os.path.exists(path):
      # Probably a deletion
      continue
    original = open(path).read()
    response = urllib2.urlopen(args.url, original)
    formatted = response.read()

    if original == formatted:
      print >> sys.stderr, '%s: formatting is correct!' % filename
      continue

    diff = difflib.unified_diff(
        original.split('\n'), formatted.split('\n'),
        os.path.join('a', filename), os.path.join('b', filename),
        lineterm='')

    if args.inplace:
      with open(path, 'w') as fh:
        fh.write(formatted)

      print >> sys.stderr, '%s: %d insertion(s), %d deletion(s)' % (
          filename,
          sum(1 for x in diff if x.startswith('+')),
          sum(1 for x in diff if x.startswith('-')))
    else:
      print '\n'.join(diff)


if __name__ == '__main__':
  main()



#!/usr/bin/python3

#  This file is part of Clementine.
#
#  Clementine is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Clementine is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Clementine.  If not, see <http://www.gnu.org/licenses/>.

from distutils import spawn
import logging
import os
import re
import subprocess
import sys
import traceback

LOGGER = logging.getLogger('macdeploy')

FRAMEWORK_SEARCH_PATH = [
    '/target', '/target/lib', '/Library/Frameworks',
    os.path.join(os.environ['HOME'], 'Library/Frameworks')
]

STRIP_PREFIX = [
    '@@HOMEBREW_PREFIX@@/opt/qt5/lib/',
    '@@HOMEBREW_CELLAR@@/qt5/5.8.0_1/lib/',
]

LIBRARY_SEARCH_PATH = ['/target', '/target/lib', '/usr/local/lib', '/sw/lib']

GSTREAMER_PLUGINS = [
    # Core plugins
    'libgstapp.dylib',
    'libgstaudioconvert.dylib',
    'libgstaudiofx.dylib',
    'libgstaudiotestsrc.dylib',
    'libgstaudioresample.dylib',
    'libgstautodetect.dylib',
    'libgstcoreelements.dylib',
    'libgstequalizer.dylib',
    'libgstgdp.dylib',
    'libgstosxaudio.dylib',
    'libgstplayback.dylib',
    'libgsttcp.dylib',
    'libgsttypefindfunctions.dylib',
    'libgstudp.dylib',
    'libgstvolume.dylib',

    # Codecs
    'libgstapetag.dylib',
    'libgstasf.dylib',
    'libgstaudioparsers.dylib',
    #'libgstfaac.dylib',
    'libgstfaad.dylib',
    'libgstflac.dylib',
    'libgstid3demux.dylib',
    'libgstisomp4.dylib',
    'libgstlame.dylib',
    'libgstlibav.dylib',
    # TODO: Bring back Musepack support.
    'libgstogg.dylib',
    'libgstopus.dylib',
    'libgstreplaygain.dylib',
    'libgstspeex.dylib',
    'libgsttaglib.dylib',
    'libgstvorbis.dylib',
    'libgstwavparse.dylib',
    'libgstxingmux.dylib',

    # HTTP src support
    'libgstsoup.dylib',

    # Icecast support
    'libgsticydemux.dylib',

    # RTSP streaming
    'libgstrtp.dylib',
    'libgstrtsp.dylib',
]

GSTREAMER_SEARCH_PATH = [
    '/usr/local/lib/gstreamer-1.0',
    '/target/lib/gstreamer-1.0',
    '/target/libexec/gstreamer-1.0',
    '/usr/local/opt/gstreamer/libexec/gstreamer-1.0',
]

QT_PLUGINS = [
    #'accessible/libqtaccessiblewidgets.dylib',
    #'codecs/libqcncodecs.dylib',
    #'codecs/libqjpcodecs.dylib',
    #'codecs/libqkrcodecs.dylib',
    #'codecs/libqtwcodecs.dylib',
    'iconengines/libqsvgicon.dylib',
    'imageformats/libqgif.dylib',
    'imageformats/libqico.dylib',
    'imageformats/libqjpeg.dylib',
    #'imageformats/libqmng.dylib',
    'imageformats/libqsvg.dylib',
    'platforms/libqcocoa.dylib',
    'styles/libqmacstyle.dylib',
]
QT_PLUGINS_SEARCH_PATH = [
    '/usr/local/opt/qt5/plugins',
    '/target/plugins',
    '/usr/local/Trolltech/Qt-4.7.0/plugins',
    '/Developer/Applications/Qt/plugins',
]

GIO_MODULES_SEARCH_PATH = [
  '/usr/local/lib/gio/modules',
  '/target/lib/gio/modules',
]

INSTALL_NAME_TOOL_APPLE = 'install_name_tool'
INSTALL_NAME_TOOL_CROSS = 'x86_64-apple-darwin-%s' % INSTALL_NAME_TOOL_APPLE
INSTALL_NAME_TOOL = INSTALL_NAME_TOOL_CROSS if spawn.find_executable(
    INSTALL_NAME_TOOL_CROSS) else INSTALL_NAME_TOOL_APPLE

OTOOL_APPLE = 'otool'
OTOOL_CROSS = 'x86_64-apple-darwin-%s' % OTOOL_APPLE
OTOOL = OTOOL_CROSS if spawn.find_executable(OTOOL_CROSS) else OTOOL_APPLE


class Error(Exception):
  pass


class CouldNotFindFrameworkError(Error):
  pass


class InstallNameToolError(Error):
  pass


class CouldNotFindXinePluginError(Error):
  pass


class CouldNotFindQtPluginError(Error):
  pass


class CouldNotFindGstreamerPluginError(Error):
  pass


class CouldNotFindGioModuleError(Error):
  pass


class CouldNotParseFrameworkNameError(Error):
  pass


if len(sys.argv) < 2:
  print('Usage: %s <bundle.app>' % sys.argv[0])

bundle_dir = sys.argv[1]

bundle_name = os.path.basename(bundle_dir).split('.')[0]

commands = []

frameworks_dir = os.path.join(bundle_dir, 'Contents', 'Frameworks')
commands.append(['mkdir', '-p', frameworks_dir])
resources_dir = os.path.join(bundle_dir, 'Contents', 'Resources')
commands.append(['mkdir', '-p', resources_dir])
plugins_dir = os.path.join(bundle_dir, 'Contents', 'PlugIns')
binary = os.path.join(bundle_dir, 'Contents', 'MacOS', bundle_name)

fixed_libraries = set()
fixed_frameworks = set()


def GetBrokenLibraries(binary):
  output = subprocess.Popen(
      [OTOOL, '-L', binary], stdout=subprocess.PIPE).communicate()[0].decode('utf-8')
  broken_libs = {'frameworks': [], 'libs': []}
  for line in [x.split(' ')[0].lstrip() for x in output.split('\n')[1:]]:
    if not line:  # skip empty lines
      continue
    if os.path.basename(binary) in line:
      continue
    elif re.match(r'^\s*/System/', line):
      continue  # System framework
    elif re.match(r'^\s*/usr/lib/', line):
      continue  # unix style system library
    elif re.match(r'Breakpad', line):
      continue  # Manually added by cmake.
    elif re.match(r'^\s*@loader_path', line):
      abs_path = os.path.join(
        os.path.dirname(binary),
        *os.path.split(line)[1:],
      )
      broken_libs['libs'].append(abs_path)
    elif re.match(r'^\s*@executable_path', line):
      # Potentially already fixed library
      relative_path = os.path.join(*line.split('/')[3:])
      if not os.path.exists(os.path.join(frameworks_dir, relative_path)):
        broken_libs['frameworks'].append(relative_path)
    elif re.search(r'\w+\.framework', line):
      broken_libs['frameworks'].append(line)
    else:
      broken_libs['libs'].append(line)

  return broken_libs


def FindFramework(path):
  for prefix in STRIP_PREFIX:
    if path.startswith(prefix):
      path = path[len(prefix):]
      break
  for search_path in FRAMEWORK_SEARCH_PATH:
    abs_path = os.path.join(search_path, path)
    if os.path.exists(abs_path):
      LOGGER.debug("Found framework '%s' in '%s'", path, search_path)
      return abs_path

  raise CouldNotFindFrameworkError(path)


def FindLibrary(path):
  if os.path.exists(path):
    return path
  for search_path in LIBRARY_SEARCH_PATH:
    abs_path = os.path.join(search_path, path)
    if os.path.exists(abs_path):
      LOGGER.debug("Found library '%s' in '%s'", path, search_path)
      return abs_path

  raise CouldNotFindFrameworkError(path)


def FixAllLibraries(broken_libs):
  for framework in broken_libs['frameworks']:
    FixFramework(framework)
  for lib in broken_libs['libs']:
    FixLibrary(lib)


def FixFramework(path):
  if path in fixed_frameworks:
    return
  else:
    fixed_frameworks.add(path)
  abs_path = FindFramework(path)
  broken_libs = GetBrokenLibraries(abs_path)
  FixAllLibraries(broken_libs)

  new_path = CopyFramework(abs_path)
  id = os.sep.join(new_path.split(os.sep)[3:])
  FixFrameworkId(new_path, id)
  for framework in broken_libs['frameworks']:
    FixFrameworkInstallPath(framework, new_path)
  for library in broken_libs['libs']:
    FixLibraryInstallPath(library, new_path)


def FixLibrary(path):
  if path in fixed_libraries or FindSystemLibrary(os.path.basename(
      path)) is not None:
    return
  else:
    fixed_libraries.add(path)
  abs_path = FindLibrary(path)
  broken_libs = GetBrokenLibraries(abs_path)
  FixAllLibraries(broken_libs)

  new_path = CopyLibrary(abs_path)
  FixLibraryId(new_path)
  for framework in broken_libs['frameworks']:
    FixFrameworkInstallPath(framework, new_path)
  for library in broken_libs['libs']:
    FixLibraryInstallPath(library, new_path)


def FixPlugin(abs_path, subdir):
  broken_libs = GetBrokenLibraries(abs_path)
  FixAllLibraries(broken_libs)

  new_path = CopyPlugin(abs_path, subdir)
  for framework in broken_libs['frameworks']:
    FixFrameworkInstallPath(framework, new_path)
  for library in broken_libs['libs']:
    FixLibraryInstallPath(library, new_path)


def FixBinary(path):
  broken_libs = GetBrokenLibraries(path)
  FixAllLibraries(broken_libs)
  for framework in broken_libs['frameworks']:
    FixFrameworkInstallPath(framework, path)
  for library in broken_libs['libs']:
    FixLibraryInstallPath(library, path)


def CopyLibrary(path):
  new_path = os.path.join(frameworks_dir, os.path.basename(path))
  args = ['cp', path, new_path]
  commands.append(args)
  commands.append(['chmod', '+w', new_path])
  LOGGER.info("Copying library '%s'", path)
  return new_path


def CopyPlugin(path, subdir):
  new_path = os.path.join(plugins_dir, subdir, os.path.basename(path))
  args = ['mkdir', '-p', os.path.dirname(new_path)]
  commands.append(args)
  args = ['cp', path, new_path]
  commands.append(args)
  commands.append(['chmod', '+w', new_path])
  LOGGER.info("Copying plugin '%s'", path)
  return new_path


def CopyFramework(src_binary):
  while os.path.islink(src_binary):
    src_binary = os.path.realpath(src_binary)

  m = re.match(r'(.*/([^/]+)\.framework)/Versions/([^/]+)/.*', src_binary)
  if not m:
    raise CouldNotParseFrameworkNameError(src_binary)

  src_base = m.group(1)
  name = m.group(2)
  version = m.group(3)

  LOGGER.info('Copying framework %s version %s', name, version)

  dest_base = os.path.join(frameworks_dir, '%s.framework' % name)
  dest_dir = os.path.join(dest_base, 'Versions', version)
  dest_binary = os.path.join(dest_dir, name)

  commands.append(['mkdir', '-p', dest_dir])
  commands.append(['cp', src_binary, dest_binary])
  commands.append(['chmod', '+w', dest_binary])

  # Copy special files from various places:
  #   QtCore has Resources/qt_menu.nib (copy to app's Resources)
  #   Sparkle has Resources/*
  #   Qt* have Resources/Info.plist
  resources_src = os.path.join(src_base, 'Resources')
  menu_nib = os.path.join(resources_src, 'qt_menu.nib')
  if os.path.exists(menu_nib):
    LOGGER.info("Copying qt_menu.nib '%s'", menu_nib)
    commands.append(['cp', '-r', menu_nib, resources_dir])
  elif os.path.exists(resources_src):
    LOGGER.info("Copying resources dir '%s'", resources_src)
    commands.append(['cp', '-r', resources_src, dest_dir])

  info_plist = os.path.join(src_base, 'Contents', 'Info.plist')
  if os.path.exists(info_plist):
    LOGGER.info("Copying special file '%s'", info_plist)
    resources_dest = os.path.join(dest_dir, 'Resources')
    commands.append(['mkdir', resources_dest])
    commands.append(['cp', '-r', info_plist, resources_dest])

  # Create symlinks in the Framework to make it look like
  # https://developer.apple.com/library/mac/documentation/MacOSX/Conceptual/BPFrameworks/Concepts/FrameworkAnatomy.html
  commands.append([
      'ln', '-sf', 'Versions/Current/%s' % name, os.path.join(dest_base, name)
  ])
  commands.append([
      'ln', '-sf', 'Versions/Current/Resources',
      os.path.join(dest_base, 'Resources')
  ])
  commands.append(
      ['ln', '-sf', version, os.path.join(dest_base, 'Versions/Current')])

  return dest_binary


def FixId(path, library_name):
  id = '@executable_path/../Frameworks/%s' % library_name
  args = [INSTALL_NAME_TOOL, '-id', id, path]
  commands.append(args)


def FixLibraryId(path):
  library_name = os.path.basename(path)
  FixId(path, library_name)


def FixFrameworkId(path, id):
  FixId(path, id)


def FixInstallPath(library_path, library, new_path):
  args = [INSTALL_NAME_TOOL, '-change', library_path, new_path, library]
  commands.append(args)


def FindSystemLibrary(library_name):
  for path in ['/lib', '/usr/lib']:
    full_path = os.path.join(path, library_name)
    if os.path.exists(full_path):
      return full_path
  return None


def FixLibraryInstallPath(library_path, library):
  system_library = FindSystemLibrary(os.path.basename(library_path))
  if system_library is None:
    new_path = '@executable_path/../Frameworks/%s' % os.path.basename(
        library_path)
    FixInstallPath(library_path, library, new_path)
  else:
    FixInstallPath(library_path, library, system_library)


def FixFrameworkInstallPath(library_path, library):
  parts = library_path.split(os.sep)
  for i, part in enumerate(parts):
    if re.match(r'\w+\.framework', part):
      full_path = os.path.join(*parts[i:])
      break
  new_path = '@executable_path/../Frameworks/%s' % full_path
  FixInstallPath(library_path, library, new_path)


def FindXinePlugin(name):
  for path in XINEPLUGIN_SEARCH_PATH:
    if os.path.exists(path):
      for dir, dirs, files in os.walk(path):
        if name in files:
          return os.path.join(dir, name)
  raise CouldNotFindXinePluginError(name)


def FindQtPlugin(name):
  for path in QT_PLUGINS_SEARCH_PATH:
    if os.path.exists(path):
      if os.path.exists(os.path.join(path, name)):
        return os.path.join(path, name)
  raise CouldNotFindQtPluginError(name)


def FindGstreamerPlugin(name):
  for path in GSTREAMER_SEARCH_PATH:
    if os.path.exists(path):
      for dir, dirs, files in os.walk(path):
        if name in files:
          return os.path.join(dir, name)
  raise CouldNotFindGstreamerPluginError(name)


def FindGioModule(name):
  for path in GIO_MODULES_SEARCH_PATH:
    if os.path.exists(path):
      for dir, dirs, files in os.walk(path):
        if name in files:
          return os.path.join(dir, name)
  raise CouldNotFindGioModuleError(name)


def main():
  logging.basicConfig(
      filename='macdeploy.log',
      level=logging.DEBUG,
      format='%(asctime)s %(levelname)-8s %(message)s')

  FixBinary(binary)

  for plugin in GSTREAMER_PLUGINS:
    FixPlugin(FindGstreamerPlugin(plugin), 'gstreamer')

  FixPlugin(FindGstreamerPlugin('gst-plugin-scanner'), '.')
  FixPlugin(FindGioModule('libgiognutls.so'), 'gio-modules')

  try:
    FixPlugin('clementine-spotifyblob', '.')
    FixPlugin('clementine-tagreader', '.')
  except:
    print('Failed to find blob: %s' % traceback.format_exc())

  for plugin in QT_PLUGINS:
    FixPlugin(FindQtPlugin(plugin), os.path.dirname(plugin))

  if len(sys.argv) <= 2:
    print('Would run %d commands:' % len(commands))
    for command in commands:
      print(' '.join(command))

    print('OK?')
    input()

  for command in commands:
    p = subprocess.Popen(command)
    os.waitpid(p.pid, 0)


if __name__ == "__main__":
  main()
